/*
popclk Version 0.7.0 - World Population Clock
Copyright (C) 2001,2002  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <ncurses.h>
#include <time.h>
#include <errno.h>
#include <sys/times.h>

int runflg;
double totpop;
double dayticks,dbltkdlta;
time_t origclk;

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s <annual_change>\n",
      pgm);
   fprintf(stderr,"Where annual_change is the "
      "annual increase or decrease\n");
   fprintf(stderr,"from -999999999999999 to "
      "999999999999999\n");
   fprintf(stderr,"Exit by pressing escape\n");
   exit(1);
   } /* putstx */

void initnc()
   {
   initscr();
   cbreak();
   noecho();
   nonl();
   intrflush(stdscr,FALSE);
   keypad(stdscr,TRUE);
   nodelay(stdscr,TRUE);
   curs_set(0);
   clear();
   refresh();
   } /* initnc */

void escape()
   {
   int ch;
   ch = getch();
   if (ch == 0x1b)
      {
      runflg = 0;
      }
   } /* escape */

void slp()
   {
   int rslt;
   struct timespec request;
   struct timespec remain;
   request.tv_sec  = (long int) 0;
   request.tv_nsec = (long int) 1;
   rslt = nanosleep(&request,&remain);
   if (rslt < 0)
      {
      fprintf(stderr,"Nanosleep error %d\n",
         errno);
      perror("Nanosleep error");
      exit(1);
      } /* nanosleep error */
   } /* slp using nanosleep */

void shwtps(ticks)
unsigned int ticks;
   {
   int ch;
   char str[32];
   sprintf(str,"Ticks per second %u", ticks);
   addstr(str);
   move(2,0);
   addstr("Press enter, escape to exit");
   refresh();
   ch = -1;
   while (ch < 0)
      {
      ch = getch();
      if (ch == 0x1b)
         {
         endwin();
         exit(0);
         } /* if escape */
      slp();
      } /* wait for key */
   clear();
   refresh();
   } /* shwtps */

int gettps()
   {
   int ch;
   unsigned int diff;
   time_t frstsec,currsec;
   clock_t prev,clk;
   struct tms t;
   /* get first date & time in seconds */
   time(&frstsec);
   currsec = frstsec;
   while (currsec == frstsec)
      {
      /* get first date & time in seconds */
      time(&currsec);
      slp();
      } /* loop until second starts */
   /* get first clock ticks since boot */
   prev = times(&t);
   frstsec = currsec;
   while (currsec == frstsec)
      {
      /* get current date & time in seconds */
      time(&currsec);
      slp();
      } /* loop for 1 second */
   /* get clock ticks since boot */
   clk = times(&t);
   if (clk < prev)
      {
      fprintf(stderr,"Negative ticks, rerun job\n");
      exit(0);
      }
   diff = clk - prev;
   return(diff);
   } /* gettps */

void count()
   {
   int rslt;
   unsigned int diff;
   double tgtpop,currpop;
   time_t clk;
   struct tms t;
   struct timespec request;
   struct timespec remain;
   char str[64];
   currpop = totpop;
   if (dbltkdlta > 0.0)
      tgtpop = totpop + 1.0;
   else
      tgtpop = totpop - 1.0;
   while (runflg)
      {
      if (dbltkdlta > 0.0
         && currpop > tgtpop) break;
      else if (dbltkdlta <= 0.0
         && currpop < tgtpop) break;
      clk = times(&t);
      if (clk < origclk)
         diff = clk + dayticks - origclk;
      else
         diff = clk - origclk;
      /* recalculate the total */
      /* population from scratch */
      currpop = dbltkdlta * diff;
      escape();
      if (runflg)
         {
         request.tv_sec  = (long int) 0;
         request.tv_nsec = (long int) 1;
         rslt = nanosleep(&request,&remain);
         if (rslt < 0)
            {
            endwin();
            fprintf(stderr,"Nanosleep error %d\n",
               errno);
            perror("Nanosleep error");
            exit(1);
            } /* nanosleep error */
         } /* if in run mode */
      else break;
      } /* wait for birth */
   totpop = currpop;
   sprintf(str,"%10.0f", totpop);
   move(2,2);
   clrtoeol();
   addstr(str);
   move(0,0);
   refresh();
   } /* count */

void runclk()
   {
   while (runflg)
      {
      count();
      escape();
      } /* display loop */
   } /* runclk */

int main(argc,argv)
int argc;
char **argv;
   {
   unsigned int tps;
   double dblanndlta,dblspy,sign;
   double dbltps,dbltpy;
   double ten,unit;
   char *p;
   struct tms t;
   if (argc != 2) putstx(*argv);
   p = *(argv+1);
   if (*p == '-')
      {
      sign = -1.0;
      p++;
      }
   else sign = 1.0;
   dblanndlta = 0.0;
   ten = 10.0;
   while (*p != '\0')
      {
      if (*p < '0' || *p > '9')
         putstx(*argv);
      unit = (double) *p - '0';
      dblanndlta = (dblanndlta * ten) + unit;
      p++;
      }
   if (dblanndlta > 999999999999999)
      putstx(*argv);
   /* annual change in population */
   dblanndlta *= sign;
   /* seconds per year */
   dblspy = 3600.0 * 24.0 * 365.25;
   /* ticks per second */
   tps = gettps();
   if (tps < 1 || tps > 999999)
      {
      fprintf(stderr,"Invalid tps %d\n", tps);
      exit(1);
      } /* if tps error */
   dbltps = (double) tps;
   /* ticks per year */
   dbltpy = dblspy * dbltps;
   /* population change per tick */
   dbltkdlta = dblanndlta / dbltpy;
   /* ticks per day */
   dayticks = 3600 * 24 * tps;
   /* initialize the curses screen */
   initnc();
   shwtps(tps);
   mvaddstr(0,0,"World Population Change");
   /* start counting now */
   origclk = times(&t);
   totpop = 0.0;
   runflg = 1;
   runclk();
   endwin();
   return(0);
   } /* main */
